package com.blogspot.nkoksharov.springevents.jgroups;

import org.jgroups.ChannelClosedException;
import org.jgroups.ChannelNotConnectedException;
import org.jgroups.JChannel;
import org.jgroups.Message;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.event.SimpleApplicationEventMulticaster;

/**
 * Alternative multicaster to org.springframework.context.event.SimpleApplicationEventMulticaster
 *
 * @author nkoksharov
 *
 */
public class JGroupsMulticaster extends SimpleApplicationEventMulticaster
		implements InitializingBean, DisposableBean, LocalEventMulticaster {

	private final Logger logger = LoggerFactory.getLogger(getClass());
	private JChannel jchannel;
	private JGroupsReceiver receiver = new JGroupsReceiver();
	private String configFile;
	private String clusterName;

	@Override
	public void multicastEvent(ApplicationEvent event) {
		if (event instanceof JGroupsEvent) {
			try {
				jchannel.send(new Message(null, null, event));
			} catch (ChannelNotConnectedException e) {
				logger.error("channel not connected", e);
			} catch (ChannelClosedException e) {
				logger.error("channel closed", e);
			}
		} else {
			super.multicastEvent(event);
		}
	}

	@Override
	public void localMulticastEvent(JGroupsEvent event) {
		super.multicastEvent(event);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		jchannel = new JChannel(configFile);
		receiver.setLocalMulticater(this);
		jchannel.setReceiver(receiver);
		jchannel.connect(clusterName);
	}

	@Override
	public void destroy() throws Exception {
		jchannel.close();
	}

	public void setConfigFile(String configFile) {
		this.configFile = configFile;
	}

	public void setClusterName(String clusterName) {
		this.clusterName = clusterName;
	}

}

